@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool-II/pool_session_context.c,v 1.27 2011/07/27 04:03:43 kitagawa Exp $
+ * $Header$
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2011	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -40,6 +40,8 @@ static bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc);
 void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 {
 	session_context = &session_context_d;
+	int node_id;
+	int i;
 
 	/* Get Process context */
 	session_context->process_context = pool_get_process_context();
@@ -68,19 +70,22 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 	/* Choose load balancing node if neccessary */
 	if (pool_config->load_balance_mode)
 	{
-		ProcessInfo *process_info = pool_get_my_process_info();
-		if (!process_info)
-		{
-			pool_error("pool_init_session_context: pool_get_my_process_info failed");
-			return;
-		}
-
-		session_context->load_balance_node_id = 
-			process_info->connection_info->load_balancing_node =
-			select_load_balancing_node();
-
-		pool_debug("selected load balancing node: %d", backend->info->load_balancing_node);
+		node_id = select_load_balancing_node();
 	}
+	else
+	{
+		node_id = STREAM? PRIMARY_NODE_ID: MASTER_NODE_ID;
+	}
+
+	session_context->load_balance_node_id = node_id;
+
+	for (i=0;i<NUM_BACKENDS;i++)
+	{
+		pool_coninfo(session_context->process_context->proc_id,
+					 pool_pool_index(), i)->load_balancing_node = node_id;
+	}
+
+	pool_debug("selected load balancing node: %d", node_id);
 
 	/* Unset query is in progress */
 	pool_unset_query_in_progress();
@@ -124,7 +129,11 @@ void pool_session_context_destroy(void)
 		pool_clear_sent_message_list();
 		free(session_context->message_list.sent_messages);
 		pool_memory_delete(session_context->memory_context, 0);
+
+		if (session_context->query_context)
+			pool_query_context_destroy(session_context->query_context);
 	}
+
 	/* XXX For now, just zap memory */
 	memset(&session_context_d, 0, sizeof(session_context_d));
 	session_context = NULL;
@@ -725,6 +734,13 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 	/* No cached data is available. Ask backend. */
 	status = do_query(MASTER(session_context->backend),
 					  "SELECT current_setting('transaction_isolation')", &res, MAJOR(session_context->backend));
+	if(status != POOL_CONTINUE)
+	{
+		pool_error("pool_get_transaction_isolation: do_query returns no rows");
+		if(res)
+			free_select_result(res);
+		return POOL_UNKNOWN;
+	}
 
 	if (res->numrows <= 0)
 	{
@@ -745,8 +761,12 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 		return POOL_UNKNOWN;
 	}
 
-	if (!strcmp(res->data[0], "read committed"))
+	if (!strcmp(res->data[0], "read uncommitted"))
+		ret = POOL_READ_UNCOMMITTED;
+	else if (!strcmp(res->data[0], "read committed"))
 		ret = POOL_READ_COMMITTED;
+	else if (!strcmp(res->data[0], "repeatable read"))
+		ret = POOL_REPEATABLE_READ;
 	else if (!strcmp(res->data[0], "serializable"))
 		ret = POOL_SERIALIZABLE;
 	else
@@ -756,7 +776,8 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 		ret = POOL_UNKNOWN;
 	}   
 
-	free_select_result(res);
+	if(res)
+		free_select_result(res);
 
 	if (ret != POOL_UNKNOWN)
 		session_context->transaction_isolation = ret;
@@ -924,3 +945,50 @@ static bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc)
 
 	return true;
 }
+
+/*
+ * Set protocol major version number
+ */
+void pool_set_major_version(int major)
+{
+	if (session_context)
+	{
+		session_context->major = major; 
+	}
+}
+
+/*
+ * Get protocol major version number
+ */
+int pool_get_major_version(void)
+{
+	if (session_context)
+	{
+		return session_context->major;
+	}
+	return PROTO_MAJOR_V3;
+}
+
+/*
+ * Set protocol minor version number
+ */
+void pool_set_minor_version(int minor)
+{
+	if (session_context)
+	{
+		session_context->minor = minor; 
+	}
+}
+
+/*
+ * Get protocol minor version number
+ */
+int pool_get_minor_version(void)
+{
+	if (session_context)
+	{
+		return session_context->minor;
+	}
+	return 0;
+}
+
